/++
 + Ogg main header module for D programming language
 +
 + Use, distribution and reproduction of this library source is 
 + governed by a BSD-STYLE souce license included with this source in 'COPYING'.
 + Please read these terms before distributing.
 +
 + Copyright(C) 2009-2012 Takuya Kurosawa
 +
 + Original code:
 +  ogg.h (libogg-1.2.2)
 +  THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2007
 +  by the Xiph.Org Foundation http://www.xiph.org/
 +/
module org.xiph.libogg.ogg;

import org.xiph.libogg.types;

struct ogg_iovec_t
{
	void* iov_base;
	size_t iov_len;
}

struct oggpack_buffer
{
	int endbyte;
	int endbit;
	ubyte* buffer;
	ubyte* ptr;
	int storage;
}

/++
 + ogg_page is used to encapsulate the data in one Ogg bitstream page
 +/
struct ogg_page
{
	ubyte* header;
	int header_len;
	ubyte* body_ptr;
	int body_len;
}

/++
 + ogg_stream_state contains the current encode/decode state of a logical Ogg bitstream
 +/
struct ogg_stream_state
{
	// bytes from packet bodies
	ubyte* body_data;

	// storage elements allocated
	int body_storage;

	// elements stored; fill mark
	int body_fill;

	// elements of fill returned
	int body_returned;

	// The values that will go to the segment table
	int* lacing_vals;

	// granulepos values for headers. Not compact this way, but it is simple coupled to the lacing fifo
	ogg_int64_t* granule_vals;

	//
	int lacing_storage;

	//
	int lacing_fill;

	//
	int lacing_packet;

	//
	int lacing_returned;

	// working space for header encode
	ubyte[282] header;

	//
	int header_fill;

	// set when we have buffered the last packet in the logical bitstream
	int e_o_s;

	// set after we've written the initial page of a logical bitstream
	int b_o_s;

	//
	int serialno;

	//
	long pageno;

	// sequence number for decode; the framing knows where there's a hole in the data,
	// but we need coupling so that the codec(which is in a separate abstraction layer)
	// also knows about the gap
	ogg_int64_t packetno;

	//
	ogg_int64_t granulepos;
}

/++
 + ogg_packet is used to encapsulate the data and metadata belonging to a single raw Ogg/Vorbis packet
 +/
struct ogg_packet
{
	ubyte* packet;
	int bytes;
	int b_o_s;
	int e_o_s;
	ogg_int64_t granulepos;

	// sequence number for decode;
	// the framing knows where there's a hole in the data,
	// but we need coupling so that the codec(which is in a separate abstraction layer)
	// also knows about the gap
	ogg_int64_t packetno;
}

struct ogg_sync_state
{
	ubyte* data;
	int storage;
	int fill;
	int returned;
	int unsynced;
	int headerbytes;
	int bodybytes;
}

extern (C)
{
	/+
	 + Ogg BITSTREAM PRIMITIVES: bitstream
	 +/
	void oggpack_writeinit(oggpack_buffer* b);
	int oggpack_writecheck(oggpack_buffer* b);
	void oggpack_writetrunc(oggpack_buffer* b, int bits);
	void oggpack_writealign(oggpack_buffer* b);
	void oggpack_writecopy(oggpack_buffer* b, void* source, int bits);
	void oggpack_reset(oggpack_buffer* b);
	void oggpack_writeclear(oggpack_buffer* b);
	void oggpack_readinit(oggpack_buffer* b, ubyte* buf,int bytes);
	void oggpack_write(oggpack_buffer* b, uint value, int bits);
	int oggpack_look(oggpack_buffer* b,int bits);
	int oggpack_look1(oggpack_buffer* b);
	void oggpack_adv(oggpack_buffer* b,int bits);
	void oggpack_adv1(oggpack_buffer* b);
	int oggpack_read(oggpack_buffer* b,int bits);
	int oggpack_read1(oggpack_buffer* b);
	int oggpack_bytes(oggpack_buffer* b);
	int oggpack_bits(oggpack_buffer* b);
	ubyte* oggpack_get_buffer(oggpack_buffer* b);

	void oggpackB_writeinit(oggpack_buffer* b);
	int oggpackB_writecheck(oggpack_buffer* b);
	void oggpackB_writetrunc(oggpack_buffer* b, int bits);
	void oggpackB_writealign(oggpack_buffer* b);
	void oggpackB_writecopy(oggpack_buffer* b, void* source, int bits);
	void oggpackB_reset(oggpack_buffer* b);
	void oggpackB_writeclear(oggpack_buffer* b);
	void oggpackB_readinit(oggpack_buffer* b, ubyte* buf, int bytes);
	void oggpackB_write(oggpack_buffer* b, uint value, int bits);
	int oggpackB_look(oggpack_buffer* b, int bits);
	int oggpackB_look1(oggpack_buffer* b);
	void oggpackB_adv(oggpack_buffer* b, int bits);
	void oggpackB_adv1(oggpack_buffer* b);
	int oggpackB_read(oggpack_buffer* b, int bits);
	int oggpackB_read1(oggpack_buffer* b);
	int oggpackB_bytes(oggpack_buffer* b);
	int oggpackB_bits(oggpack_buffer* b);
	ubyte* oggpackB_get_buffer(oggpack_buffer* b);


	/+
	 + Ogg BITSTREAM PRIMITIVES: encoding
	 +/
	int ogg_stream_packetin(ogg_stream_state* os, ogg_packet* op);
	int ogg_stream_iovecin(ogg_stream_state* os, ogg_iovec_t* iov, int count, long e_o_s, ogg_int64_t granulepos);
	int ogg_stream_pageout(ogg_stream_state* os, ogg_page* og);
	int ogg_stream_pageout_fill(ogg_stream_state* os, ogg_page* og, int nfill);
	int ogg_stream_flush(ogg_stream_state* os, ogg_page* og);


	/+
	 + Ogg BITSTREAM PRIMITIVES: decoding
	 +/
	int ogg_sync_init(ogg_sync_state* oy);
	int ogg_sync_clear(ogg_sync_state* oy);
	int ogg_sync_reset(ogg_sync_state* oy);
	int ogg_sync_destroy(ogg_sync_state* oy);
	int ogg_sync_check(ogg_sync_state* oy);

	byte* ogg_sync_buffer(ogg_sync_state* oy, int size);
	int ogg_sync_wrote(ogg_sync_state* oy, int bytes);
	int ogg_sync_pageseek(ogg_sync_state* oy,ogg_page* og);
	int ogg_sync_pageout(ogg_sync_state* oy, ogg_page* og);
	int ogg_stream_pagein(ogg_stream_state* os, ogg_page* og);
	int ogg_stream_packetout(ogg_stream_state* os,ogg_packet* op);
	int ogg_stream_packetpeek(ogg_stream_state* os,ogg_packet* op);


	/+
	 + Ogg BITSTREAM PRIMITIVES: general
	 +/
	int ogg_stream_init(ogg_stream_state* os,int serialno);
	int ogg_stream_clear(ogg_stream_state* os);
	int ogg_stream_reset(ogg_stream_state* os);
	int ogg_stream_reset_serialno(ogg_stream_state* os,int serialno);
	int ogg_stream_destroy(ogg_stream_state* os);
	int ogg_stream_check(ogg_stream_state* os);
	int ogg_stream_eos(ogg_stream_state* os);

	void ogg_page_checksum_set(ogg_page* og);

	int ogg_page_version(const(ogg_page)* og);
	int ogg_page_continued(const(ogg_page)* og);
	int ogg_page_bos(const(ogg_page)* og);
	int ogg_page_eos(const(ogg_page)* og);
	ogg_int64_t  ogg_page_granulepos(const(ogg_page)* og);
	int ogg_page_serialno(const(ogg_page)* og);
	int ogg_page_pageno(const(ogg_page)* og);
	int ogg_page_packets(const(ogg_page)* og);

	void ogg_packet_clear(ogg_packet* op);
}