module jp.ku6.tetris.VirtualPad;

import std.traits;
import jp.ku6.windows.game.input;

/++
 + 仮想コントローラ
 +/
class VirtualPad
{
	/// ボタン
	enum Button
	{
		UP,
		DOWN,
		LEFT,
		RIGHT,
		A,
		B,
	}

	// 機器状態の更新
	void refresh()
	{
		Input input = Input.instance;

		// 入力デバイスの状態を更新
		input.refresh();

		// 前の入力情報をバックアップ
		uint oldButtonPress = buttonPress_;

		// 情報をクリア
		buttonPress_ = 0;
		buttonPush_ = 0;

		foreach (button; EnumMembers!(VirtualPad.Button))
		{
			if (input.keyboard.isPress(KEYBOARD_KEY_MAP[button]))
				buttonPress_ |= 1 << button;

			foreach (pad; input.joypads)
			{
				if (pad.isPress(JOYPAD_BUTTON_MAP[button]))
					buttonPress_ |= 1 << button;
			}
		}

		// ボタンを押した瞬間かチェック
		buttonPush_ = buttonPress_ & (~oldButtonPress);
	}

	/++
	 + ボタンが押されているか判定
	 +
	 + 現在ボタンが押されているか判定する
	 +
	 + Params:
	 +  button = 判定するボタン
	 +
	 + Returns:
	 +  true = 押されている
	 +  false = 押されていない
	 +/
	const
	bool isPress(Button button)
	{
		return (buttonPress_ & (1 << button)) != 0;
	}

	/++
	 + ボタンが押された瞬間か判定
	 +
	 + 1秒間に60フレーム等だと、普通に押しただけでisPressは複数フレームの間ONが返される
	 + この関数は押された最初のフレームだけONを返す
	 +
	 + Params:
	 +  button = 判定するボタン
	 +
	 + Returns:
	 +  true = 押されている
	 +  false = 押されていない
	 +/
	const
	bool isPush(Button button)
	{
		return (buttonPush_ & (1 << button)) != 0;
	}

private:

	// キーボードのキーマップ
	immutable static
	Keyboard.Key[EnumMembers!(VirtualPad.Button).length] KEYBOARD_KEY_MAP =
	[
		Keyboard.Key.UP,
		Keyboard.Key.DOWN,
		Keyboard.Key.LEFT,
		Keyboard.Key.RIGHT,
		Keyboard.Key.X,
		Keyboard.Key.Z,
	];

	// ジョイパッドのキーマップ
	immutable static
	Joypad.Button[EnumMembers!(VirtualPad.Button).length] JOYPAD_BUTTON_MAP =
	[
		Joypad.Button.UP,
		Joypad.Button.DOWN,
		Joypad.Button.LEFT,
		Joypad.Button.RIGHT,
		Joypad.Button.A,
		Joypad.Button.B,
	];

	// 現在ボタンが押されているかの情報
	uint buttonPress_;

	// ボタンを押された瞬間の情報
	uint buttonPush_;
}
