module jp.ku6.tetris.winmain;

import core.runtime;
import core.thread;
import std.stream;
import std.math;
import jp.ku6.windows.win32api.windows;
import jp.ku6.windows.win32api.winuser;
import jp.ku6.windows.game.application;
import jp.ku6.windows.game.graphics;
import jp.ku6.windows.game.input;
import jp.ku6.windows.game.audio;
import jp.ku6.tetris.game;

version(Win64)
{
	immutable GAME_NAME = "Full HD Tetris 64bit version";
}
else
{
	immutable GAME_NAME = "Full HD Tetris";
}

extern (Windows)
int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	if (Runtime.initialize() == 0)
		return 1;
	scope(exit) Runtime.terminate();

	Application.initialize(hInstance, 960, 540, GAME_NAME, GAME_NAME, null, null, null);
	scope (exit) Application.terminate();

	if (Application.instance.isStarted(GAME_NAME))
	{
		MessageBoxW(null, "既に起動しています"w.ptr, "Error\0"w.ptr, MB_OK | MB_ICONEXCLAMATION);
		return 2;
	}

	int result;
	try
	{
		Graphics.initialize(1920, 1080);
		scope (exit) Graphics.terminate();

		Input.initialize(1920, 1080);
		scope (exit) Input.terminate();

		Audio.initialize();
		scope (exit) Audio.terminate();

		auto game = new Tetris;
		scope (exit) delete game;

		while (Application.instance().isValid())
		{
			game.mainLoop();
		}
	}
	catch (Throwable o)
	{
		wstring message = std.utf.toUTF16(o.toString() ~ "\0");
		MessageBoxW(null, message.ptr, "Error\0"w.ptr, MB_OK | MB_ICONEXCLAMATION);
		result = 3;

		// メッセージループを処理しきる
		Application.instance().exit();
		while (Application.instance().isValid()) {}
	}

	return result;
}
